import React, {Fragment} from 'react';
import PropTypes from 'prop-types';
import Select, {components} from 'react-select';
import {
    FormGroup,
    Label,
    Tooltip
} from 'reactstrap';

const customComponents = {
    NoOptionsMessage: props => (
        <Tooltip content="noMessageOption">
            <components.NoOptionsMessage {...props}>
                No se encontraron resultados&nbsp;
            </components.NoOptionsMessage>
            &nbsp;
        </Tooltip>
    )
};

const SelectFilter = ({
                         label,
                         control,
                         defaultOption,
                         disabled,
                         filterOption,
                         getValue,
                         onChange,
                         onInputChange,
                         options,
                         placeholder,
                         value,
                         required
                     }) => (
    <FormGroup>
        <Label>
            &nbsp;
            {label}
            &nbsp;
        </Label>
        &nbsp;
        <Fragment>
            <Select
                id={control}
                value={getValue ? getValue(value, options) : value}
                defaultValue={defaultOption ? {value: '', label: '[Todos]'} : false}
                onChange={onChange}
                onInputChange={onInputChange}
                options={options}
                components={customComponents}
                filterOption={filterOption}
                isSearchable
                isClearable
                isDisabled={disabled}
                placeholder={placeholder}
                clearValueText="Limpiar Filtro"
            />
            {!disabled && required && (
                <input
                    className="sr-only"
                    tabIndex={-1}
                    value={value ? value.value : ''}
                    required
                />
            )}
        </Fragment>
    </FormGroup>
);

SelectFilter.propTypes = {
    label: PropTypes.string.isRequired,
    control: PropTypes.string.isRequired,
    options: PropTypes.arrayOf(PropTypes.shape({})).isRequired,
    onChange: PropTypes.func.isRequired,
    value: PropTypes.oneOfType([
        PropTypes.string,
        PropTypes.number
    ]),
    onInputChange: PropTypes.func,
    getValue: PropTypes.func,
    defaultOption: PropTypes.bool,
    placeholder: PropTypes.string,
    disabled: PropTypes.bool,
    filterOption: PropTypes.oneOfType([
        PropTypes.any
    ]),
    required: PropTypes.bool
};

SelectFilter.defaultProps = {
    defaultOption: false,
    placeholder: '[Seleccione]',
    value: '',
    onInputChange: null,
    getValue: null,
    disabled: false,
    filterOption: null,
    required: true
};

export default SelectFilter;
