import React, {Component} from "react";
import Alert from "reactstrap/lib/Alert";
import {Button} from "reactstrap";
import {map} from "lodash";
import {FontAwesomeIcon} from "@fortawesome/react-fontawesome";
import {
    faBackspace,
    faEdit,
    faSortUp,
    faSortDown,
    faTimes
} from "@fortawesome/free-solid-svg-icons";
import PropTypes from "prop-types";
import {Col, Row} from "reactstrap";
import {
    errorStyle,
    buttonStyle,
    buttonIconStyle,
    iconStyle,
    tableStyle,
    headerContentStyle,
    headerInContentStyle,
    textHeaderStyle,
    noPadding
} from '../styles';
import SpecialFilters from "./SpecialFilters";

class TableComponent extends Component {
    static propTypes = {
        data: PropTypes.shape().isRequired,
        head: PropTypes.shape().isRequired,
        query: PropTypes.shape().isRequired,
        error: PropTypes.shape().isRequired,
        handleQuery: PropTypes.func.isRequired,
        onUpdate: PropTypes.func.isRequired,
        onDeactivate: PropTypes.func.isRequired,
        styles: PropTypes.shape({
            tableStyle: PropTypes.string,
            headerContentStyle: PropTypes.string,
            buttonIconStyle: PropTypes.shape(),
            textHeaderStyle: PropTypes.shape()
        })
    }
    static defaultProps = {
        styles: {
            tableStyle: tableStyle,
            headerContentStyle: headerContentStyle,
            buttonIconStyle:buttonIconStyle,
            textHeaderStyle: textHeaderStyle
        }
    }

    constructor(props) {
        super(props);
        this.state = {
            dropdownOpen: false,
            inputValue: ''
        }
    }


    shipQuery = (query) => {
        this.props.handleQuery(query)
    }


    handleChange(e) {
        this.setState({inputValue: e.value})
    }

    handleThWidth(key){
        return (this.props.head[key] === "Numero"?'10%':null) || (this.props.head[key] === "Activo"? '10%': null)
    }

    render() {
        const {data, head, query, error, styles} = this.props;
        return (
            <div className="container-fluid" style={{...noPadding}} >
                <Row style={{display: 'block'}}>
                    {error.anErrorOccurred &&
                    <Alert color="danger" style={errorStyle}>
                        {error.message}
                    </Alert>}
                </Row>
                <Row >
                    <Col style={{...noPadding}}>
                        <SpecialFilters
                            headers={head}
                            onSearch={ (e,id) => this.shipQuery({
                                        fname: id,
                                        fvalue: this.state.inputValue,
                                        start: 0
                                    })
                            }
                            onChangeView={() => {
                                this.shipQuery((query.search[0].active === 'S') ? ({
                                    start: 0,
                                    active: 'N'
                                }) : ({start: 0, active: 'S'}))
                            }}
                            active={query.search[0].active}
                            onCleanSearch={() => {
                                this.shipQuery({9999: '9999'})
                            }}
                            onChange={e=>this.handleChange(e)}
                            seeker={this.state.inputValue}
                        />
                    </Col>
                </Row>
                <Row>
                    {this.props.data !== false && (
                        <table className={styles.tableStyle}>
                            <thead>
                                <tr>
                                    {map(data[0], (value, key) => {
                                        if(head[key]){
                                            return(
                                                <th style={{...noPadding}} scope="col" width={this.handleThWidth(key)}>
                                                    <div className={styles.headerContentStyle}>
                                                        <div style={headerInContentStyle}>
                                                            <p style={styles.textHeaderStyle}>
                                                                {head[key] === 'Numero'? 'Nro' : head[key] }
                                                            </p>
                                                        </div>
                                                        <div>
                                                            <Button id="caret"  style={styles.buttonIconStyle} onClick={() => {
                                                                this.shipQuery(query.search[0].infoOrder === 'DESC' ? ({infoOrder: 'ASC', orderby:key}) : ({infoOrder: 'DESC', orderby:key}))
                                                            }}>
                                                                <FontAwesomeIcon
                                                                    color={ query.search[0].orderby === key ? '#1266F1' : null}
                                                                    icon={query.search[0].infoOrder === 'ASC' ? faSortUp : faSortDown}
                                                                    style={{iconStyle, ...noPadding, color: query.search[0].orderby === key ? 'primary' : null }}
                                                                />
                                                            </Button>
                                                        </div>
                                                    </div>
                                                </th>
                                            )
                                        }
                                    })}
                                    <th style={{...noPadding}} scope="col" className={'col-md-1'}>
                                        <FontAwesomeIcon
                                            icon={faTimes}
                                            style={{iconStyle, marginRight: '1rem', ...noPadding}}
                                        />
                                    </th>
                                    <th style={{...noPadding}} scope="col" className={'col-md-1'} align={'center'}>
                                        <FontAwesomeIcon
                                            icon={faEdit}
                                            style={{iconStyle, marginRight: '1rem', ...noPadding}}/>
                                    </th>
                                </tr>
                            </thead>
                            <tbody>
                            {map(data, (objj) => {
                                return (
                                        <tr>
                                        {map(objj, (obj, key) => {
                                            if (head[key]) {
                                                return (<td style={{textAlign:head[key] === 'Activo'? 'center': 'start'}}>{obj}</td>)
                                            }
                                        })}
                                        <td align={'center'}>
                                            <Button
                                                style={{...buttonStyle}}
                                                color="danger"
                                                onClick={() => {
                                                    this.props.onDeactivate(objj.id, objj.number, objj.active)
                                                    this.shipQuery({reload: 'reload'})
                                                }}>
                                                <FontAwesomeIcon icon={faBackspace} style={iconStyle}/>
                                            </Button>
                                        </td>
                                        <td align={'center'}>
                                            <Button
                                                style={buttonStyle}
                                                color="success"
                                                onClick={() => {
                                                    this.props.onUpdate(objj.id)
                                                    this.shipQuery({reload: 'reload'})
                                                }}>
                                                <FontAwesomeIcon icon={faEdit} style={iconStyle}/>
                                            </Button>
                                        </td>
                                        </tr>
                                )
                            })}
                            </tbody>
                        </table>
                    )}
                </Row>
            </div>
        );
    }
}

export {TableComponent}

