import React, {Component} from 'react';
import OptionsService from '../services/network/fetchSelectOptions'
import PropTypes from "prop-types";
import Loading from "../Loading";
import {map, includes} from 'lodash';
import {TableComponent, PaginationComponent} from '../common/components/index'

class TableView extends Component {
    static propTypes = {
        table: PropTypes.shape().isRequired,
        token: PropTypes.string.isRequired
    }

    constructor(props) {
        super(props);
        this.state = {
            query: this.props.table.query,
            result: false,
            size: 0,
            total: 0,
            loading: true,
            error: ({anErrorOccurred: false, message: null}),
            page: 0,
            table: this.props.table
        }
    }

    componentDidUpdate(prevProps, prevState, snapshot) {
        if (this.state.loading ) {
            if (this.props.table.query.endpoint !== this.state.query.endpoint){
                OptionsService.fetchTable(this.props.token, this.props.table.query).then(obj => this.setState(obj))
            } else {
                OptionsService.fetchTable(this.props.token, this.state.query).then(obj => this.setState(obj))
            }

        }
    }

    componentDidMount(prevProps) {
        if (this.state.loading && this.props.table.loadingModule === false) {
            if ((includes(this.props.table.query.endpoint.split('/'), this.props.table.moduleName))) {
                OptionsService.fetchTable(this.props.token, this.state.query).then(obj => this.setState(obj))
            } else {
                this.setState({
                    table: this.props.table,
                    query: this.props.table.query
                })
            }
        }

    }

    handlerOnSearch(query) {
        this.setState(prevState => {
            let state = prevState
            map(query, (value, key) => {
                switch (key) {
                    case '9999':
                        for (const key in state.query.search[0]) {
                            switch (key){
                                case 'active':
                                    state.query.search[0][key] = 'S'
                                    break
                                case 'infoOrder':
                                    state.query.search[0][key] = 'ASC'
                                    break
                                default:
                                    state.query.search[0][key] = null
                                    break
                            }
                        }
                        state = ({
                            query: this.props.table.query,
                            result: false,
                            loading: true
                        })
                        break
                    case 'reload':
                        state.loading = true
                        break
                    case 'start':
                        state.page = value
                        state.query.search[0][key] = value
                        state.loading = true
                        break
                    default:
                        state.query.search[0][key] = value;
                        state.loading = true
                        break
                }
            })
            return (state)
        })
    }

    render() {
        return (
            <div>
                {this.state.loading ? <Loading/> :
                    <div>
                        <TableComponent data={this.state.result}
                                        head={this.state.table.headers[0]}
                                        handleQuery={(e) => {
                                            this.handlerOnSearch(e)
                                        }}
                                        query={this.state.query}
                                        error={this.state.error}
                                        onUpdate={this.state.table.functions[0]}
                                        onDeactivate={this.state.table.functions[1]}
                                        styles={this.props.table.styles}
                        />
                        <PaginationComponent
                            handleQuery={(e) => {
                                this.handlerOnSearch(e)
                            }}
                            sizePagination={this.state.size}
                            total={this.state.total}
                            current={this.state.page}
                            pageCount={this.state.pageCount}
                        />
                    </div>
                }
            </div>
        )
    }
}

export default TableView;
